unit MainUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls;

type
  TMainForm = class(TForm)
    Memo1: TMemo;
    Panel1: TPanel;
    StartButton: TButton;
    AbortButton: TButton;
    procedure FormCreate(Sender: TObject);
    procedure StartButtonClick(Sender: TObject);
    procedure AbortButtonClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

{$R *.DFM}

uses
  Math, AIOWDG;

type
  TGrowlLogResetThread = class(TThread)
  protected
    procedure Execute; override;
  end;

var
  CardNum: Integer;



procedure WriteFileAndFlush(FileName, FileBody: String);
var
  hFile: THandle;
  BytesTransferred: LongWord;

begin
  hFile := CreateFile(PChar(FileName), GENERIC_READ or GENERIC_WRITE, FILE_SHARE_READ or FILE_SHARE_WRITE, nil, CREATE_ALWAYS, 0, 0);
  FileName := '';

  WriteFile(hFile, FileBody[1], Length(FileBody), BytesTransferred, nil);
  //Some SATA drives won't complete the flush when commanded, but will complete the flush in order to complete a read.
  //So in order to complete the flush to disk, we command a flush, then go back and read the contents.
  FlushFileBuffers(hFile);
  SetFilePointer(hFile, 0, nil, FILE_BEGIN);
  ReadFile(hFile, FileBody[1], Length(FileBody), BytesTransferred, nil);
  FileBody := '';

  CloseHandle(hFile);
end;

{ TMainForm }

procedure TMainForm.FormCreate(Sender: TObject);
begin
  CardNum := 0;
end;

procedure TMainForm.StartButtonClick(Sender: TObject);
var
  WorkerThread: TGrowlLogResetThread;

begin
  StartButton.Enabled := False;

  WorkerThread := TGrowlLogResetThread.Create(True);
  WorkerThread.FreeOnTerminate := True;
  WorkerThread.Resume;

  AbortButton.Enabled := True;
end;

procedure TMainForm.AbortButtonClick(Sender: TObject);
begin
  AbortButton.Enabled := False;

  AbortRequest(CardNum);

  StartButton.Enabled := True;
end;

{ TGrowlLogResetThread }

procedure TGrowlLogResetThread.Execute;
var
  FN, Content: String;
  ST: TSystemTime;

begin
  ReturnValue := WaitForIRQ(CardNum);
  if ReturnValue <> ERROR_SUCCESS then Exit; //It was cancelled or something.

  //If we get to here, then a growl "IRQ" has just occurred.
  GetLocalTime(ST);
  with ST do Content := Format('Rebooting in response to growl, at %d:%.2d:%.2d on %d/%d/%d.', [ wHour, wMinute, wSecond, wDay, wMonth, wYear ]);
  with ST do FN := Format('.\%.4d%.2d%.2d %.2d%.2d%.2d Growl.txt', [ wYear, wMonth, wDay, wHour, wMinute, wSecond ]);

  WriteFileAndFlush(FN, Content);

  EmergencyReboot;
end;

end.
